DAT programName  byte "LoadToEeprom_150206b", 0
' I like to have the name of the program displayed in a terminal window.
CON{{
  Program to load PASM sections of "Servo32v9" and "F32" to EEPROM.

  There is also code to generate the a call table for "F32" which
  does not require being compiled with the PASM section of the code.
  
}}
CON{

  150206a Tighty up in preparation to post to the forum.
  06b Remove header file.
     
}
CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5000000 'Header#CRYSTAL_FREQUENCY

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MICROSECOND = CLK_FREQ / 1_000_000
  MILLISECOND = CLK_FREQ / 1_000

  I2C_CLOCK = 28
  I2C_DATA = 29

  DEBUG_BAUD = 115_200
  XBEE_BAUD = 115_200
  DEBUG_RX_PIN = 31
  DEBUG_TX_PIN = 30

  #0, DEBUG_COM ', XBEE_COM

CON '' EEPROM Constants

  F32_EEPROM_START = $8000
  SERVO_EEPROM_START = $876C ' entered after previously running this program
  SERIAL_EEPROM_START = $8964 ' next PASM section will start here
  EMPTY_EEPROM_START = SERIAL_EEPROM_START

CON

''     Parallax Serial Terminal
''    Control Character Constants

  CS = 16  ''CS: Clear Screen      
  CE = 11  ''CE: Clear to End of line     
  CB = 12  ''CB: Clear lines Below 

  HM =  1  ''HM: HoMe cursor       
  PC =  2  ''PC: Position Cursor in x,y          
  PX = 14  ''PX: Position cursor in X         
  PY = 15  ''PY: Position cursor in Y         

  NL = 13  ''NL: New Line        
  LF = 10  ''LF: Line Feed       
  ML =  3  ''ML: Move cursor Left          
  MR =  4  ''MR: Move cursor Right         
  MU =  5  ''MU: Move cursor Up          
  MD =  6  ''MD: Move cursor Down
  TB =  9  ''TB: TaB          
  BS =  8  ''BS: BackSpace          
           
  BP =  7  ''BP: BeeP speaker          

OBJ

  Servo : "Servo32v9NoRampToEeprom"                             ' uses 1 cog
  F32[2] : "F32ToEeprom"       ' uses 2 cog
 
  'Header : "HalloweenHexQsHeader150205a"
  'ComHeader : "HeaderHexapodCom140209a"
  Com : "Serial4Port140918a"                            ' uses 1 cog

  'Format : "StrFmt"
  Ee : "dwd_PropellerEeprom"
  'Nunchuck : "jm_nunchuk_ez_v3"

  'Leds : "HexEyesX150131a"
  
PUB Main | legIndex, first[3], last[3], size[3], eeprom[3]

  
  Com.Init
  Com.AddPort(DEBUG_COM, DEBUG_RX_PIN, DEBUG_TX_PIN, -1, -1, 0, 0, DEBUG_BAUD)
  '**141014b Com.AddPort(XBEE_COM, Header#XBEE_RX_PIN, Header#XBEE_TX_PIN, -1, -1, 0, 0, XBEE_BAUD)
  Com.Start                                             'Start the ports    
  
  repeat
    NewLine
    Com.Str(DEBUG_COM, string("Press any key to begin."))
    NewLine
    Com.Str(DEBUG_COM, string("Warning, this program will write to the upper EEPROM."))
    waitcnt(clkfreq / 4 + cnt)
    result := Com.RxCheck(DEBUG_COM)     
  while result == -1

  Com.Tx(DEBUG_COM, CS)
  Com.Tx(DEBUG_COM, HM)
  'Com.E
  waitcnt(clkfreq / 20 + cnt)

  NewLine
  Com.Str(DEBUG_COM, string("F32 cmdCallTable = "))
  DumpLongs(F32.Call_ptr, 21)
  
  NewLine
  NewLine
  Com.Str(DEBUG_COM, string("The above output should be copied into F32."))
  NewLine
  Com.Str(DEBUG_COM, string("The above output is the call table which may be used"))
  NewLine
  Com.Str(DEBUG_COM, string("without being compiled with the associated PASM code."))
  NewLine
  Com.Str(DEBUG_COM, string("Press any key to continue."))
  repeat
    waitcnt(clkfreq / 4 + cnt)
    result := Com.RxCheck(DEBUG_COM)     
  while result == -1
  
  NewLine
  Com.Str(DEBUG_COM, string("F32 PASM Starts (RAM) $"))
  first[0] := F32.GetFirst
  Com.Hex(DEBUG_COM, first, 4)
  NewLine
  Com.Str(DEBUG_COM, string("F32 PASM Last Long (RAM) $"))
  last[0] := F32.GetLast
  Com.Hex(DEBUG_COM, last, 4)
  NewLine
  Com.Str(DEBUG_COM, string("F32 PASM Size $"))
  size[0] := (last[0] - first[0]) + 4
  Com.Hex(DEBUG_COM, size[0], 4)
  NewLine
  Com.Str(DEBUG_COM, string("Ee.FromRam($"))
  Com.Hex(DEBUG_COM, first[0], 4)
  Com.Str(DEBUG_COM, string(", $"))
  Com.Hex(DEBUG_COM, first[0] + size[0] - 1, 4)
  Com.Str(DEBUG_COM, string(", $"))
  Com.Hex(DEBUG_COM, F32_EEPROM_START, 4)
  Com.Tx(DEBUG_COM, ")")
  
  '' Uncomment the line below to write PASM section of the servo object to EEPROM.
  ''Ee.FromRam(first[0], first[0] + size[0] - 1, F32_EEPROM_START) 

  NewLine 
  Com.Str(DEBUG_COM, string("Servo PASM Starts (RAM) $"))
  first[1] := Servo.GetFirst
  Com.Hex(DEBUG_COM, first[1], 4)
  NewLine
  Com.Str(DEBUG_COM, string("Servo PASM Last Long (RAM) $"))
  last[1] := Servo.GetLast
  Com.Hex(DEBUG_COM, last[1], 4)
  NewLine
  Com.Str(DEBUG_COM, string("Servo PASM Size $"))
  size[1] := (last[1] - first[1]) + 4
  Com.Hex(DEBUG_COM, size[1], 4)
  eeprom[0] := F32_EEPROM_START + size[0] 'Header#
  eeprom[1] := eeprom[0] + size[1]
  NewLine
  Com.Str(DEBUG_COM, string("Ee.FromRam($"))
  Com.Hex(DEBUG_COM, first[1], 4)
  Com.Str(DEBUG_COM, string(", $"))
  Com.Hex(DEBUG_COM, first[1] + size[1] - 1, 4)
  Com.Str(DEBUG_COM, string(", $"))
  Com.Hex(DEBUG_COM, eeprom[0], 4)
  Com.Tx(DEBUG_COM, ")")
  
  '' Uncomment the line below to write PASM section of F32 to EEPROM.
  ''Ee.FromRam(first[1], first[1] + size[1] - 1, eeprom[0])
  
  NewLine
  Com.Str(DEBUG_COM, string("Start of servo PASM in EEPROM: eeprom[0] = $"))
  Com.Hex(DEBUG_COM, eeprom[0], 4)
  NewLine
  Com.Str(DEBUG_COM, string("Start of unused area in EEPROM: eeprom[1] = $"))
  Com.Hex(DEBUG_COM, eeprom[1], 4)

  NewLine
  Com.Str(DEBUG_COM, string("Write down the above EEPROM addresses to use in the program"))
  NewLine
  Com.Str(DEBUG_COM, string("which will read these PASM sections."))
  NewLine
  NewLine
  Com.Str(DEBUG_COM, string("Press any key to compare EEPROM data with RAM data."))
  repeat
    waitcnt(clkfreq / 4 + cnt)
    result := Com.RxCheck(DEBUG_COM)     
  while result == -1


  result := CompareLongs(first[0], F32_EEPROM_START, size[0] / 4) 

  result := CompareLongs(first[1], eeprom[0], size[1] / 4)   
  
  Com.Str(DEBUG_COM, string("End of program."))

  repeat

PUB DumpLongs(localPtr, localSize)
'' Display longs formatted for inclusion in "F32FromEeprom".

  localSize--
  repeat result from 0 to localSize
    NewLine
    Com.Str(DEBUG_COM, FindString(@callTableText, result))
    Com.Tx(DEBUG_COM, CE)
    Com.Tx(DEBUG_COM, PX)
    Com.Tx(DEBUG_COM, 25)
    Com.Str(DEBUG_COM, string("long $"))
    Com.Hex(DEBUG_COM, long[localPtr][result], 8)
    
PUB CompareLongs(localPtr, eepromPtr, localSize) | localLong, rowCount

  rowCount := 0
  NewLine
  Com.Str(DEBUG_COM, string("Comparing RAM with EEPROM."))
  NewLine
  Com.Str(DEBUG_COM, string("RAM address = $"))
  Com.Hex(DEBUG_COM, localPtr, 4)
  NewLine
  Com.Str(DEBUG_COM, string("EEPROM address = $"))
  Com.Hex(DEBUG_COM, eepromPtr, 4)
  NewLine
  Com.Str(DEBUG_COM, string("Longs to compare = $"))
  Com.Hex(DEBUG_COM, localSize, 4)
  NewLine
  
  repeat localSize
    ifnot rowCount++ // 4
      NewLine
      Com.Str(DEBUG_COM, string("<$"))
      Com.Hex(DEBUG_COM, localPtr, 4)
      Com.Str(DEBUG_COM, string("&$"))
      Com.Hex(DEBUG_COM, eepromPtr, 4)
      Com.Str(DEBUG_COM, string(">"))

    Ee.ToRam(@localLong, @localLong + 3, eepromPtr)
    Com.Str(DEBUG_COM, string("|$"))
    Com.Hex(DEBUG_COM, long[localPtr], 8)
    if long[localPtr] == localLong
      Com.Str(DEBUG_COM, string("==$"))
    else
      Com.Str(DEBUG_COM, string(7, "<>$"))
      result++
    Com.Hex(DEBUG_COM, localLong, 8)  
    localPtr += 4
    eepromPtr += 4

  NewLine
  Com.Str(DEBUG_COM, string("The compare method found "))
  Com.Dec(DEBUG_COM, result)
  Com.Str(DEBUG_COM, string(" unmatched longs."))
  NewLine 
  if result
    Com.Str(DEBUG_COM, string(7, " Failure!!! ", 7))
  else
    Com.Str(DEBUG_COM, string(" Success!!! "))
      
DAT
callTableText           byte "cmdFAdd", 0
                        byte "cmdFSub", 0
                        byte "cmdFMul", 0
                        byte "cmdFDiv", 0
                        byte "cmdFFloat", 0
                        byte "cmdFTruncRound", 0
                        byte "cmdUintTrunc", 0
                        byte "cmdFSqr", 0
                        byte "cmdFCmp", 0
                        byte "cmdFSin", 0
                        byte "cmdFCos", 0
                        byte "cmdFTan", 0
                        byte "cmdFLog2", 0
                        byte "cmdFExp2", 0
                        byte "cmdFPow", 0
                        byte "cmdFFrac", 0
                        byte "cmdFMod", 0
                        byte "cmdASinCos", 0
                        byte "cmdATan2", 0
                        byte "cmdCeil", 0
                        byte "cmdFloor", 0

PRI NewLine                                      
'' Clear the end of the current line and then
'' start a new line.

  Com.Tx(DEBUG_COM, CE)
  Com.Tx(DEBUG_COM, 13)
  result := 1

PUB FindString(firstStr, stringIndex)      
'' Finds start address of one string in a list
'' of string. "firstStr" is the address of 
'' string #0 in the list. "stringIndex"
'' indicates which of the strings in the list
'' the method is to find.

  result := firstStr 
  repeat while stringIndex    
    repeat while byte[result++]  
    stringIndex--
             